@@ -620,8 +620,10 @@ get_profile_name_from_folder_tree (EShellView *shell_view,
 				CamelSettings *settings;
 
 				service = CAMEL_SERVICE (selected_store);
-				settings = camel_service_get_settings (service);
+
+				settings = camel_service_ref_settings (service);
 				g_object_get (settings, "profile", &profile, NULL);
+				g_object_unref (settings);
 
 				if (pstore && profile)
 					*pstore = g_object_ref (selected_store);
@@ -652,20 +654,23 @@ action_folder_size_cb (GtkAction *action,
 	gchar *profile;
 	CamelSession *session;
 	CamelStore *store = NULL;
-	CamelMapiSettings *mapi_settings;
 
 	profile = get_profile_name_from_folder_tree (shell_view, NULL, &store);
 	if (profile && store) {
+		CamelSettings *settings;
 		ESourceRegistry *registry;
 		ESource *source;
 
-		mapi_settings = CAMEL_MAPI_SETTINGS (camel_service_get_settings (CAMEL_SERVICE (store)));
 		session = camel_service_get_session (CAMEL_SERVICE (store));
 		registry = e_mail_session_get_registry (E_MAIL_SESSION (session));
 		source = e_source_registry_ref_source (registry, camel_service_get_uid (CAMEL_SERVICE (store)));
 
+		settings = camel_service_ref_settings (CAMEL_SERVICE (store));
+
 		e_mapi_config_utils_run_folder_size_dialog (
-			registry, source, mapi_settings);
+			registry, source, CAMEL_MAPI_SETTINGS (settings));
+
+		g_object_unref (settings);
 
 		g_object_unref (source); 
 	}
@@ -709,7 +714,6 @@ action_folder_permissions_mail_cb (GtkAction *action,
 	GtkWindow *parent;
 	CamelStore *store = NULL;
 	CamelMapiStore *mapi_store;
-	CamelNetworkSettings *network_settings;
 	CamelStoreInfo *si;
 
 	profile = get_profile_name_from_folder_tree (shell_view, &folder_path, &store);
@@ -720,9 +724,6 @@ action_folder_permissions_mail_cb (GtkAction *action,
 	g_return_if_fail (mapi_store != NULL);
 	g_return_if_fail (folder_path != NULL);
 
-	network_settings = CAMEL_NETWORK_SETTINGS (camel_service_get_settings (CAMEL_SERVICE (store)));
-	g_return_if_fail (network_settings != NULL);
-
 	shell_window = e_shell_view_get_shell_window (shell_view);
 	parent = GTK_WINDOW (shell_window);
 
@@ -733,14 +734,17 @@ action_folder_permissions_mail_cb (GtkAction *action,
 		CamelMapiStoreInfo *msi = (CamelMapiStoreInfo *) si;
 		ESourceRegistry *registry = e_shell_get_registry (e_shell_window_get_shell (shell_window));
 		ESource *source;
+		CamelSettings *settings;
 
 		source = e_source_registry_ref_source (registry, camel_service_get_uid (CAMEL_SERVICE (store)));
 		g_return_if_fail (source != NULL);
 
+		settings = camel_service_ref_settings (CAMEL_SERVICE (store));
+
 		e_mapi_edit_folder_permissions (parent,
 			registry,
 			source,
-			CAMEL_MAPI_SETTINGS (network_settings),
+			CAMEL_MAPI_SETTINGS (settings),
 			camel_service_get_display_name (CAMEL_SERVICE (store)),
 			folder_path,
 			msi->folder_id,
@@ -750,6 +754,8 @@ action_folder_permissions_mail_cb (GtkAction *action,
 			msi->foreign_username,
 			FALSE);
 
+		g_object_unref (settings);
+
 		g_object_unref (source);
 	}
 
@@ -1057,6 +1063,7 @@ action_folder_permissions_source_cb (GtkAction *action,
 
 	extension_name = e_source_camel_get_extension_name ("mapi");
 	extension = e_source_get_extension (parent_source, extension_name);
+
 	settings = e_source_camel_get_settings (extension);
 
 	e_mapi_edit_folder_permissions (NULL,
@@ -1449,6 +1456,7 @@ struct EMapiFolderStructureData
 	ESource *source;
 	ESource *child_source;
 	ESourceRegistry *registry;
+	ESourceConfig *config;
 };
 
 static void
@@ -1461,7 +1469,10 @@ e_mapi_folder_structure_data_free (gpointer ptr)
 
 	e_mapi_folder_free_list (fsd->folders);
 	g_object_unref (fsd->tree_view);
-	g_object_unref (fsd->source);
+	if (fsd->source)
+		g_object_unref (fsd->source);
+	if (fsd->config)
+		g_object_unref (fsd->config);
 	g_object_unref (fsd->child_source);
 	g_object_unref (fsd->registry);
 	g_free (fsd);
@@ -1508,7 +1519,6 @@ e_mapi_download_folder_structure_thread (GObject *source_obj,
 	ESourceCamel *extension;
 	EMapiConnection *conn;
 	CamelSettings *settings;
-	CamelMapiSettings *mapi_settings;
 
 	g_return_if_fail (fsd != NULL);
 	g_return_if_fail (fsd->tree_view != NULL);
@@ -1521,13 +1531,13 @@ e_mapi_download_folder_structure_thread (GObject *source_obj,
 	g_return_if_fail (e_source_has_extension (source, extension_name));
 
 	extension = e_source_get_extension (source, extension_name);
+
 	settings = e_source_camel_get_settings (extension);
-	mapi_settings = CAMEL_MAPI_SETTINGS (settings);
 
 	conn = e_mapi_config_utils_open_connection_for (NULL,
 		fsd->registry,
 		source,
-		mapi_settings,
+		CAMEL_MAPI_SETTINGS (settings),
 		cancellable,
 		perror);
 
@@ -1544,12 +1554,29 @@ e_mapi_download_folder_structure_thread (GObject *source_obj,
 		g_object_unref (conn);
 }
 
-static gboolean
-e_mapi_invoke_folder_structure_download_idle (gpointer user_data)
+static void
+tree_view_mapped_cb (GObject *tree_view)
 {
-	struct EMapiFolderStructureData *fsd = user_data;
+	const struct EMapiFolderStructureData *old_fsd = g_object_get_data (tree_view, "mapi-fsd-pointer");
+	struct EMapiFolderStructureData *fsd;
+	GtkTreeViewColumn *column;
+	ESource *parent_source;
 
-	g_return_val_if_fail (fsd != NULL, FALSE);
+	g_return_if_fail (old_fsd != NULL);
+
+	parent_source = e_source_config_get_collection_source (old_fsd->config);
+	g_return_if_fail (parent_source != NULL);
+
+	fsd = g_new0 (struct EMapiFolderStructureData, 1);
+	fsd->folder_type = old_fsd->folder_type;
+	fsd->folders = NULL;
+	fsd->tree_view = g_object_ref (old_fsd->tree_view);
+	fsd->source = g_object_ref (parent_source);
+	fsd->child_source = g_object_ref (old_fsd->child_source);
+	fsd->registry = g_object_ref (old_fsd->registry);
+
+	column = gtk_tree_view_get_column (GTK_TREE_VIEW (tree_view), 0);
+	gtk_tree_view_column_set_title (column, e_source_get_display_name (parent_source));
 
 	e_mapi_config_utils_run_in_thread_with_feedback (e_mapi_config_utils_get_widget_toplevel_window (fsd->tree_view),
 		G_OBJECT (fsd->source),
@@ -1558,8 +1585,6 @@ e_mapi_invoke_folder_structure_download_idle (gpointer user_data)
 		e_mapi_download_folder_structure_idle,
 		fsd,
 		e_mapi_folder_structure_data_free);
-
-	return FALSE;
 }
 
 void
@@ -1651,7 +1676,6 @@ e_mapi_config_utils_insert_widgets (ESourceConfigBackend *backend,
 		GtkTreeViewColumn *column;
 		GtkTreeStore *tree_store;
 		GtkWidget *tree_view, *scrolled_window;
-		ESource *parent_source;
 
 		content_grid = GTK_GRID (gtk_grid_new ());
 		gtk_grid_set_row_spacing (content_grid, 2);
@@ -1664,9 +1688,8 @@ e_mapi_config_utils_insert_widgets (ESourceConfigBackend *backend,
 
 		tree_store = gtk_tree_store_new (NUM_COLS, G_TYPE_STRING, G_TYPE_UINT64, G_TYPE_POINTER);
 
-		parent_source = e_source_config_get_collection_source (config);
 		renderer = gtk_cell_renderer_text_new ();
-		column = gtk_tree_view_column_new_with_attributes (e_source_get_display_name (parent_source), renderer, "text", NAME_COL, NULL);
+		column = gtk_tree_view_column_new_with_attributes ("", renderer, "text", NAME_COL, NULL);
 		tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
 		gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
 		g_object_set (tree_view, "expander-column", column, "headers-visible", TRUE, NULL);
@@ -1690,11 +1713,12 @@ e_mapi_config_utils_insert_widgets (ESourceConfigBackend *backend,
 			fsd->folder_type = folder_type;
 			fsd->folders = NULL;
 			fsd->tree_view = g_object_ref (tree_view);
-			fsd->source = g_object_ref (parent_source);
+			fsd->config = g_object_ref (config);
 			fsd->child_source = g_object_ref (scratch_source);
 			fsd->registry = g_object_ref (e_source_config_get_registry (config));
 
-			g_idle_add (e_mapi_invoke_folder_structure_download_idle, fsd);
+			g_signal_connect_after (tree_view, "map", G_CALLBACK (tree_view_mapped_cb), NULL);
+			g_object_set_data_full (G_OBJECT (tree_view), "mapi-fsd-pointer", fsd, e_mapi_folder_structure_data_free);
 		}
 
 		gtk_widget_set_hexpand (GTK_WIDGET (content_grid), TRUE);
